@@ -7,13 +7,12 @@ from datetime import datetime
 
 import pytest
 import requests
-import six
 
 import docker
 from .. import helpers
 from ..helpers import assert_cat_socket_detached_with_keys
 from ..helpers import ctrl_with
-from ..helpers import requires_api_version
+from ..helpers import requires_api_version, skip_if_desktop
 from .base import BaseAPIIntegrationTest
 from .base import TEST_IMG
 from docker.constants import IS_WINDOWS_PLATFORM
@@ -35,7 +34,7 @@ class ListContainersTest(BaseAPIIntegrationTest):
         assert len(retrieved) == 1
         retrieved = retrieved[0]
         assert 'Command' in retrieved
-        assert retrieved['Command'] == six.text_type('true')
+        assert retrieved['Command'] == 'true'
         assert 'Image' in retrieved
         assert re.search(r'alpine:.*', retrieved['Image'])
         assert 'Status' in retrieved
@@ -104,13 +103,11 @@ class CreateContainerTest(BaseAPIIntegrationTest):
         self.client.start(container3_id)
         assert self.client.wait(container3_id)['StatusCode'] == 0
 
-        logs = self.client.logs(container3_id)
-        if six.PY3:
-            logs = logs.decode('utf-8')
-        assert '{0}_NAME='.format(link_env_prefix1) in logs
-        assert '{0}_ENV_FOO=1'.format(link_env_prefix1) in logs
-        assert '{0}_NAME='.format(link_env_prefix2) in logs
-        assert '{0}_ENV_FOO=1'.format(link_env_prefix2) in logs
+        logs = self.client.logs(container3_id).decode('utf-8')
+        assert f'{link_env_prefix1}_NAME=' in logs
+        assert f'{link_env_prefix1}_ENV_FOO=1' in logs
+        assert f'{link_env_prefix2}_NAME=' in logs
+        assert f'{link_env_prefix2}_ENV_FOO=1' in logs
 
     def test_create_with_restart_policy(self):
         container = self.client.create_container(
@@ -125,8 +122,8 @@ class CreateContainerTest(BaseAPIIntegrationTest):
         self.client.wait(id)
         with pytest.raises(docker.errors.APIError) as exc:
             self.client.remove_container(id)
-        err = exc.value.explanation
-        assert 'You cannot remove ' in err
+        err = exc.value.explanation.lower()
+        assert 'stop the container before' in err
         self.client.remove_container(id, force=True)
 
     def test_create_container_with_volumes_from(self):
@@ -218,6 +215,20 @@ class CreateContainerTest(BaseAPIIntegrationTest):
 
         self.client.kill(id)
 
+    @requires_api_version('1.41')
+    def test_create_with_cgroupns(self):
+        host_config = self.client.create_host_config(cgroupns='private')
+
+        container = self.client.create_container(
+            image=TEST_IMG,
+            command=['sleep', '60'],
+            host_config=host_config,
+        )
+        self.tmp_containers.append(container)
+
+        res = self.client.inspect_container(container)
+        assert 'private' == res['HostConfig']['CgroupnsMode']
+
     def test_group_id_ints(self):
         container = self.client.create_container(
             TEST_IMG, 'id -G',
@@ -227,9 +238,7 @@ class CreateContainerTest(BaseAPIIntegrationTest):
         self.client.start(container)
         self.client.wait(container)
 
-        logs = self.client.logs(container)
-        if six.PY3:
-            logs = logs.decode('utf-8')
+        logs = self.client.logs(container).decode('utf-8')
         groups = logs.strip().split(' ')
         assert '1000' in groups
         assert '1001' in groups
@@ -244,9 +253,7 @@ class CreateContainerTest(BaseAPIIntegrationTest):
         self.client.start(container)
         self.client.wait(container)
 
-        logs = self.client.logs(container)
-        if six.PY3:
-            logs = logs.decode('utf-8')
+        logs = self.client.logs(container).decode('utf-8')
 
         groups = logs.strip().split(' ')
         assert '1000' in groups
@@ -467,16 +474,13 @@ class CreateContainerTest(BaseAPIIntegrationTest):
     def test_create_with_device_cgroup_rules(self):
         rule = 'c 7:128 rwm'
         ctnr = self.client.create_container(
-            TEST_IMG, 'cat /sys/fs/cgroup/devices/devices.list',
-            host_config=self.client.create_host_config(
+            TEST_IMG, 'true', host_config=self.client.create_host_config(
                 device_cgroup_rules=[rule]
             )
         )
         self.tmp_containers.append(ctnr)
         config = self.client.inspect_container(ctnr)
         assert config['HostConfig']['DeviceCgroupRules'] == [rule]
-        self.client.start(ctnr)
-        assert rule in self.client.logs(ctnr).decode('utf-8')
 
     def test_create_with_uts_mode(self):
         container = self.client.create_container(
@@ -494,7 +498,7 @@ class CreateContainerTest(BaseAPIIntegrationTest):
 )
 class VolumeBindTest(BaseAPIIntegrationTest):
     def setUp(self):
-        super(VolumeBindTest, self).setUp()
+        super().setUp()
 
         self.mount_dest = '/mnt'
 
@@ -515,10 +519,7 @@ class VolumeBindTest(BaseAPIIntegrationTest):
             TEST_IMG,
             ['ls', self.mount_dest],
         )
-        logs = self.client.logs(container)
-
-        if six.PY3:
-            logs = logs.decode('utf-8')
+        logs = self.client.logs(container).decode('utf-8')
         assert self.filename in logs
         inspect_data = self.client.inspect_container(container)
         self.check_container_data(inspect_data, True)
@@ -534,15 +535,34 @@ class VolumeBindTest(BaseAPIIntegrationTest):
             TEST_IMG,
             ['ls', self.mount_dest],
         )
-        logs = self.client.logs(container)
+        logs = self.client.logs(container).decode('utf-8')
 
-        if six.PY3:
-            logs = logs.decode('utf-8')
         assert self.filename in logs
 
         inspect_data = self.client.inspect_container(container)
         self.check_container_data(inspect_data, False)
 
+    @skip_if_desktop()
+    def test_create_with_binds_rw_rshared(self):
+        container = self.run_with_volume_propagation(
+            False,
+            'rshared',
+            TEST_IMG,
+            ['touch', os.path.join(self.mount_dest, self.filename)],
+        )
+        inspect_data = self.client.inspect_container(container)
+        self.check_container_data(inspect_data, True, 'rshared')
+        container = self.run_with_volume_propagation(
+            True,
+            'rshared',
+            TEST_IMG,
+            ['ls', self.mount_dest],
+        )
+        logs = self.client.logs(container).decode('utf-8')
+        assert self.filename in logs
+        inspect_data = self.client.inspect_container(container)
+        self.check_container_data(inspect_data, False, 'rshared')
+
     @requires_api_version('1.30')
     def test_create_with_mounts(self):
         mount = docker.types.Mount(
@@ -554,9 +574,7 @@ class VolumeBindTest(BaseAPIIntegrationTest):
             host_config=host_config
         )
         assert container
-        logs = self.client.logs(container)
-        if six.PY3:
-            logs = logs.decode('utf-8')
+        logs = self.client.logs(container).decode('utf-8')
         assert self.filename in logs
         inspect_data = self.client.inspect_container(container)
         self.check_container_data(inspect_data, True)
@@ -573,9 +591,7 @@ class VolumeBindTest(BaseAPIIntegrationTest):
             host_config=host_config
         )
         assert container
-        logs = self.client.logs(container)
-        if six.PY3:
-            logs = logs.decode('utf-8')
+        logs = self.client.logs(container).decode('utf-8')
         assert self.filename in logs
         inspect_data = self.client.inspect_container(container)
         self.check_container_data(inspect_data, False)
@@ -602,7 +618,7 @@ class VolumeBindTest(BaseAPIIntegrationTest):
         assert mount['Source'] == mount_data['Name']
         assert mount_data['RW'] is True
 
-    def check_container_data(self, inspect_data, rw):
+    def check_container_data(self, inspect_data, rw, propagation='rprivate'):
         assert 'Mounts' in inspect_data
         filtered = list(filter(
             lambda x: x['Destination'] == self.mount_dest,
@@ -612,6 +628,7 @@ class VolumeBindTest(BaseAPIIntegrationTest):
         mount_data = filtered[0]
         assert mount_data['Source'] == self.mount_origin
         assert mount_data['RW'] == rw
+        assert mount_data['Propagation'] == propagation
 
     def run_with_volume(self, ro, *args, **kwargs):
         return self.run_container(
@@ -629,12 +646,29 @@ class VolumeBindTest(BaseAPIIntegrationTest):
             **kwargs
         )
 
+    def run_with_volume_propagation(self, ro, propagation, *args, **kwargs):
+        return self.run_container(
+            *args,
+            volumes={self.mount_dest: {}},
+            host_config=self.client.create_host_config(
+                binds={
+                    self.mount_origin: {
+                        'bind': self.mount_dest,
+                        'ro': ro,
+                        'propagation': propagation
+                    },
+                },
+                network_mode='none'
+            ),
+            **kwargs
+        )
+
 
 class ArchiveTest(BaseAPIIntegrationTest):
     def test_get_file_archive_from_container(self):
         data = 'The Maid and the Pocket Watch of Blood'
         ctnr = self.client.create_container(
-            TEST_IMG, 'sh -c "echo {0} > /vol1/data.txt"'.format(data),
+            TEST_IMG, f'sh -c "echo {data} > /vol1/data.txt"',
             volumes=['/vol1']
         )
         self.tmp_containers.append(ctnr)
@@ -645,15 +679,14 @@ class ArchiveTest(BaseAPIIntegrationTest):
             for d in strm:
                 destination.write(d)
             destination.seek(0)
-            retrieved_data = helpers.untar_file(destination, 'data.txt')
-            if six.PY3:
-                retrieved_data = retrieved_data.decode('utf-8')
+            retrieved_data = helpers.untar_file(destination, 'data.txt')\
+                .decode('utf-8')
             assert data == retrieved_data.strip()
 
     def test_get_file_stat_from_container(self):
         data = 'The Maid and the Pocket Watch of Blood'
         ctnr = self.client.create_container(
-            TEST_IMG, 'sh -c "echo -n {0} > /vol1/data.txt"'.format(data),
+            TEST_IMG, f'sh -c "echo -n {data} > /vol1/data.txt"',
             volumes=['/vol1']
         )
         self.tmp_containers.append(ctnr)
@@ -672,9 +705,7 @@ class ArchiveTest(BaseAPIIntegrationTest):
             test_file.seek(0)
             ctnr = self.client.create_container(
                 TEST_IMG,
-                'cat {0}'.format(
-                    os.path.join('/vol1/', os.path.basename(test_file.name))
-                ),
+                f"cat {os.path.join('/vol1/', os.path.basename(test_file.name))}",
                 volumes=['/vol1']
             )
             self.tmp_containers.append(ctnr)
@@ -683,9 +714,6 @@ class ArchiveTest(BaseAPIIntegrationTest):
         self.client.start(ctnr)
         self.client.wait(ctnr)
         logs = self.client.logs(ctnr)
-        if six.PY3:
-            logs = logs.decode('utf-8')
-            data = data.decode('utf-8')
         assert logs.strip() == data
 
     def test_copy_directory_to_container(self):
@@ -700,9 +728,7 @@ class ArchiveTest(BaseAPIIntegrationTest):
             self.client.put_archive(ctnr, '/vol1', test_tar)
         self.client.start(ctnr)
         self.client.wait(ctnr)
-        logs = self.client.logs(ctnr)
-        if six.PY3:
-            logs = logs.decode('utf-8')
+        logs = self.client.logs(ctnr).decode('utf-8')
         results = logs.strip().split()
         assert 'a.py' in results
         assert 'b.py' in results
@@ -723,7 +749,7 @@ class RenameContainerTest(BaseAPIIntegrationTest):
         if version == '1.5.0':
             assert name == inspect['Name']
         else:
-            assert '/{0}'.format(name) == inspect['Name']
+            assert f'/{name}' == inspect['Name']
 
 
 class StartContainerTest(BaseAPIIntegrationTest):
@@ -829,7 +855,7 @@ class LogsTest(BaseAPIIntegrationTest):
     def test_logs(self):
         snippet = 'Flowering Nights (Sakuya Iyazoi)'
         container = self.client.create_container(
-            TEST_IMG, 'echo {0}'.format(snippet)
+            TEST_IMG, f'echo {snippet}'
         )
         id = container['Id']
         self.tmp_containers.append(id)
@@ -837,13 +863,13 @@ class LogsTest(BaseAPIIntegrationTest):
         exitcode = self.client.wait(id)['StatusCode']
         assert exitcode == 0
         logs = self.client.logs(id)
-        assert logs == (snippet + '\n').encode(encoding='ascii')
+        assert logs == f"{snippet}\n".encode(encoding='ascii')
 
     def test_logs_tail_option(self):
         snippet = '''Line1
 Line2'''
         container = self.client.create_container(
-            TEST_IMG, 'echo "{0}"'.format(snippet)
+            TEST_IMG, f'echo "{snippet}"'
         )
         id = container['Id']
         self.tmp_containers.append(id)
@@ -856,19 +882,19 @@ Line2'''
     def test_logs_streaming_and_follow(self):
         snippet = 'Flowering Nights (Sakuya Iyazoi)'
         container = self.client.create_container(
-            TEST_IMG, 'echo {0}'.format(snippet)
+            TEST_IMG, f'echo {snippet}'
         )
         id = container['Id']
         self.tmp_containers.append(id)
         self.client.start(id)
-        logs = six.binary_type()
+        logs = b''
         for chunk in self.client.logs(id, stream=True, follow=True):
             logs += chunk
 
         exitcode = self.client.wait(id)['StatusCode']
         assert exitcode == 0
 
-        assert logs == (snippet + '\n').encode(encoding='ascii')
+        assert logs == f"{snippet}\n".encode(encoding='ascii')
 
     @pytest.mark.timeout(5)
     @pytest.mark.skipif(os.environ.get('DOCKER_HOST', '').startswith('ssh://'),
@@ -876,12 +902,12 @@ Line2'''
     def test_logs_streaming_and_follow_and_cancel(self):
         snippet = 'Flowering Nights (Sakuya Iyazoi)'
         container = self.client.create_container(
-            TEST_IMG, 'sh -c "echo \\"{0}\\" && sleep 3"'.format(snippet)
+            TEST_IMG, f'sh -c "echo \\"{snippet}\\" && sleep 3"'
         )
         id = container['Id']
         self.tmp_containers.append(id)
         self.client.start(id)
-        logs = six.binary_type()
+        logs = b''
 
         generator = self.client.logs(id, stream=True, follow=True)
         threading.Timer(1, generator.close).start()
@@ -889,12 +915,12 @@ Line2'''
         for chunk in generator:
             logs += chunk
 
-        assert logs == (snippet + '\n').encode(encoding='ascii')
+        assert logs == f"{snippet}\n".encode(encoding='ascii')
 
     def test_logs_with_dict_instead_of_id(self):
         snippet = 'Flowering Nights (Sakuya Iyazoi)'
         container = self.client.create_container(
-            TEST_IMG, 'echo {0}'.format(snippet)
+            TEST_IMG, f'echo {snippet}'
         )
         id = container['Id']
         self.tmp_containers.append(id)
@@ -902,12 +928,12 @@ Line2'''
         exitcode = self.client.wait(id)['StatusCode']
         assert exitcode == 0
         logs = self.client.logs(container)
-        assert logs == (snippet + '\n').encode(encoding='ascii')
+        assert logs == f"{snippet}\n".encode(encoding='ascii')
 
     def test_logs_with_tail_0(self):
         snippet = 'Flowering Nights (Sakuya Iyazoi)'
         container = self.client.create_container(
-            TEST_IMG, 'echo "{0}"'.format(snippet)
+            TEST_IMG, f'echo "{snippet}"'
         )
         id = container['Id']
         self.tmp_containers.append(id)
@@ -921,7 +947,7 @@ Line2'''
     def test_logs_with_until(self):
         snippet = 'Shanghai Teahouse (Hong Meiling)'
         container = self.client.create_container(
-            TEST_IMG, 'echo "{0}"'.format(snippet)
+            TEST_IMG, f'echo "{snippet}"'
         )
 
         self.tmp_containers.append(container)
@@ -931,7 +957,7 @@ Line2'''
         logs_until_1 = self.client.logs(container, until=1)
         assert logs_until_1 == b''
         logs_until_now = self.client.logs(container, datetime.now())
-        assert logs_until_now == (snippet + '\n').encode(encoding='ascii')
+        assert logs_until_now == f"{snippet}\n".encode(encoding='ascii')
 
 
 class DiffTest(BaseAPIIntegrationTest):
@@ -1097,7 +1123,7 @@ class PortTest(BaseAPIIntegrationTest):
 
             ip, host_port = port_binding['HostIp'], port_binding['HostPort']
 
-            port_binding = port if not protocol else port + "/" + protocol
+            port_binding = port if not protocol else f"{port}/{protocol}"
             assert ip == port_bindings[port_binding][0]
             assert host_port == port_bindings[port_binding][1]
 
@@ -1117,7 +1143,7 @@ class ContainerTopTest(BaseAPIIntegrationTest):
         self.client.start(container)
         res = self.client.top(container)
         if not IS_WINDOWS_PLATFORM:
-            assert res['Titles'] == [u'PID', u'USER', u'TIME', u'COMMAND']
+            assert res['Titles'] == ['PID', 'USER', 'TIME', 'COMMAND']
         assert len(res['Processes']) == 1
         assert res['Processes'][0][-1] == 'sleep 60'
         self.client.kill(container)
@@ -1135,7 +1161,7 @@ class ContainerTopTest(BaseAPIIntegrationTest):
 
         self.client.start(container)
         res = self.client.top(container, '-eopid,user')
-        assert res['Titles'] == [u'PID', u'USER']
+        assert res['Titles'] == ['PID', 'USER']
         assert len(res['Processes']) == 1
         assert res['Processes'][0][10] == 'sleep 60'
 
@@ -1222,10 +1248,10 @@ class AttachContainerTest(BaseAPIIntegrationTest):
         sock = self.client.attach_socket(container, ws=False)
         assert sock.fileno() > -1
 
-    def test_run_container_reading_socket(self):
+    def test_run_container_reading_socket_http(self):
         line = 'hi there and stuff and things, words!'
         # `echo` appends CRLF, `printf` doesn't
-        command = "printf '{0}'".format(line)
+        command = f"printf '{line}'"
         container = self.client.create_container(TEST_IMG, command,
                                                  detach=True, tty=False)
         self.tmp_containers.append(container)
@@ -1242,12 +1268,33 @@ class AttachContainerTest(BaseAPIIntegrationTest):
         data = read_exactly(pty_stdout, next_size)
         assert data.decode('utf-8') == line
 
+    @pytest.mark.xfail(condition=bool(os.environ.get('DOCKER_CERT_PATH', '')),
+                       reason='DOCKER_CERT_PATH not respected for websockets')
+    def test_run_container_reading_socket_ws(self):
+        line = 'hi there and stuff and things, words!'
+        # `echo` appends CRLF, `printf` doesn't
+        command = f"printf '{line}'"
+        container = self.client.create_container(TEST_IMG, command,
+                                                 detach=True, tty=False)
+        self.tmp_containers.append(container)
+
+        opts = {"stdout": 1, "stream": 1, "logs": 1}
+        pty_stdout = self.client.attach_socket(container, opts, ws=True)
+        self.addCleanup(pty_stdout.close)
+
+        self.client.start(container)
+
+        data = pty_stdout.recv()
+        assert data.decode('utf-8') == line
+
+    @pytest.mark.timeout(10)
     def test_attach_no_stream(self):
         container = self.client.create_container(
             TEST_IMG, 'echo hello'
         )
         self.tmp_containers.append(container)
         self.client.start(container)
+        self.client.wait(container, condition='not-running')
         output = self.client.attach(container, stream=False, logs=True)
         assert output == 'hello\n'.encode(encoding='ascii')
 
@@ -1382,7 +1429,7 @@ class GetContainerStatsTest(BaseAPIIntegrationTest):
         response = self.client.stats(container, stream=0)
         self.client.kill(container)
 
-        assert type(response) == dict
+        assert isinstance(response, dict)
         for key in ['read', 'networks', 'precpu_stats', 'cpu_stats',
                     'memory_stats', 'blkio_stats']:
             assert key in response
@@ -1395,7 +1442,7 @@ class GetContainerStatsTest(BaseAPIIntegrationTest):
             self.client.start(container)
             stream = self.client.stats(container)
             for chunk in stream:
-                assert type(chunk) == dict
+                assert isinstance(chunk, dict)
                 for key in ['read', 'network', 'precpu_stats', 'cpu_stats',
                             'memory_stats', 'blkio_stats']:
                     assert key in chunk
@@ -1509,7 +1556,7 @@ class LinkTest(BaseAPIIntegrationTest):
 
         # Remove link
         linked_name = self.client.inspect_container(container2_id)['Name'][1:]
-        link_name = '%s/%s' % (linked_name, link_alias)
+        link_name = f'{linked_name}/{link_alias}'
         self.client.remove_container(link_name, link=True)
 
         # Link is gone
